@@ -1,22 +1,22 @@
 /*
 
-Copyright (C) 1996-2017 John W. Eaton
+Copyright (C) 1996-2018 John W. Eaton
 
 This file is part of Octave.
 
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
+Octave is free software: you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
+Octave is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
+<https://www.gnu.org/licenses/>.
 
 */
 
@@ -137,34 +137,39 @@ DEFNDCATOP_FN (fs_cs, float_scalar, complex, float_array,
                float_complex_array, concat)
 
 void
-install_fs_fcs_ops (void)
+install_fs_fcs_ops (octave::type_info& ti)
 {
-  INSTALL_BINOP (op_add, octave_float_scalar, octave_float_complex, add);
-  INSTALL_BINOP (op_sub, octave_float_scalar, octave_float_complex, sub);
-  INSTALL_BINOP (op_mul, octave_float_scalar, octave_float_complex, mul);
-  INSTALL_BINOP (op_div, octave_float_scalar, octave_float_complex, div);
-  INSTALL_BINOP (op_pow, octave_float_scalar, octave_float_complex, pow);
-  INSTALL_BINOP (op_ldiv, octave_float_scalar, octave_float_complex, ldiv);
-  INSTALL_BINOP (op_lt, octave_float_scalar, octave_float_complex, lt);
-  INSTALL_BINOP (op_le, octave_float_scalar, octave_float_complex, le);
-  INSTALL_BINOP (op_eq, octave_float_scalar, octave_float_complex, eq);
-  INSTALL_BINOP (op_ge, octave_float_scalar, octave_float_complex, ge);
-  INSTALL_BINOP (op_gt, octave_float_scalar, octave_float_complex, gt);
-  INSTALL_BINOP (op_ne, octave_float_scalar, octave_float_complex, ne);
-  INSTALL_BINOP (op_el_mul, octave_float_scalar, octave_float_complex, el_mul);
-  INSTALL_BINOP (op_el_div, octave_float_scalar, octave_float_complex, el_div);
-  INSTALL_BINOP (op_el_pow, octave_float_scalar, octave_float_complex, el_pow);
-  INSTALL_BINOP (op_el_ldiv, octave_float_scalar, octave_float_complex,
-                 el_ldiv);
-  INSTALL_BINOP (op_el_and, octave_float_scalar, octave_float_complex, el_and);
-  INSTALL_BINOP (op_el_or, octave_float_scalar, octave_float_complex, el_or);
+  INSTALL_BINOP_TI (ti, op_add, octave_float_scalar, octave_float_complex, add);
+  INSTALL_BINOP_TI (ti, op_sub, octave_float_scalar, octave_float_complex, sub);
+  INSTALL_BINOP_TI (ti, op_mul, octave_float_scalar, octave_float_complex, mul);
+  INSTALL_BINOP_TI (ti, op_div, octave_float_scalar, octave_float_complex, div);
+  INSTALL_BINOP_TI (ti, op_pow, octave_float_scalar, octave_float_complex, pow);
+  INSTALL_BINOP_TI (ti, op_ldiv, octave_float_scalar, octave_float_complex, ldiv);
+  INSTALL_BINOP_TI (ti, op_lt, octave_float_scalar, octave_float_complex, lt);
+  INSTALL_BINOP_TI (ti, op_le, octave_float_scalar, octave_float_complex, le);
+  INSTALL_BINOP_TI (ti, op_eq, octave_float_scalar, octave_float_complex, eq);
+  INSTALL_BINOP_TI (ti, op_ge, octave_float_scalar, octave_float_complex, ge);
+  INSTALL_BINOP_TI (ti, op_gt, octave_float_scalar, octave_float_complex, gt);
+  INSTALL_BINOP_TI (ti, op_ne, octave_float_scalar, octave_float_complex, ne);
+  INSTALL_BINOP_TI (ti, op_el_mul, octave_float_scalar, octave_float_complex,
+                    el_mul);
+  INSTALL_BINOP_TI (ti, op_el_div, octave_float_scalar, octave_float_complex,
+                    el_div);
+  INSTALL_BINOP_TI (ti, op_el_pow, octave_float_scalar, octave_float_complex,
+                    el_pow);
+  INSTALL_BINOP_TI (ti, op_el_ldiv, octave_float_scalar, octave_float_complex,
+                    el_ldiv);
+  INSTALL_BINOP_TI (ti, op_el_and, octave_float_scalar, octave_float_complex,
+                    el_and);
+  INSTALL_BINOP_TI (ti, op_el_or, octave_float_scalar, octave_float_complex,
+                    el_or);
 
-  INSTALL_CATOP (octave_float_scalar, octave_float_complex, fs_fcs);
-  INSTALL_CATOP (octave_scalar, octave_float_complex, s_fcs);
-  INSTALL_CATOP (octave_float_scalar, octave_complex, fs_cs);
+  INSTALL_CATOP_TI (ti, octave_float_scalar, octave_float_complex, fs_fcs);
+  INSTALL_CATOP_TI (ti, octave_scalar, octave_float_complex, s_fcs);
+  INSTALL_CATOP_TI (ti, octave_float_scalar, octave_complex, fs_cs);
 
-  INSTALL_ASSIGNCONV (octave_float_scalar, octave_float_complex,
-                      octave_float_complex_matrix);
-  INSTALL_ASSIGNCONV (octave_scalar, octave_float_complex,
-                      octave_complex_matrix);
+  INSTALL_ASSIGNCONV_TI (ti, octave_float_scalar, octave_float_complex,
+                         octave_float_complex_matrix);
+  INSTALL_ASSIGNCONV_TI (ti, octave_scalar, octave_float_complex,
+                         octave_complex_matrix);
 }
